# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"find_backward": "00_Core.ipynb",
         "find_data": "00_Core.ipynb",
         "create_file_path": "00_Core.ipynb",
         "download_data": "00_Core.ipynb",
         "save_authentification_session": "01_Selenium.ipynb",
         "set_auth_session_with_selenium": "01_Selenium.ipynb",
         "add_auth_session_web_driver": "01_Selenium.ipynb",
         "load_json": "01_Selenium.ipynb",
         "MetaID": "03_Structure.ipynb",
         "MongoObj": "03_Structure.ipynb",
         "update_settings_requirements": "05_cli.ipynb"}

modules = ["core.py",
           "selenium.py",
           "structure.py",
           "cli.py"]

doc_url = "https://Pyth314.github.io/kutility/"

git_url = "https://github.com/Pyth314/kutility/tree/master/"

def custom_doc_links(name): return None
